' -----------------------------------------------------------------------------
' Projekt  : Selstndige Frequenzausgabe von Timer1 ber PinD.5 (X20-Pin2)
' Hardware : Platine 5283 und Zubehr. Jumper gem Schaltplan AVRKit.
' Aufgabe  : Die Ausgabe eines Tons kann sehr elegant gelst werden. Timer1
'            bietet die Mglichkeit, direkt ein Signal auf PortD.5 auszugeben,
'            und zwar unbeeinflut vom laufenden Programms. Das spart Zeit und
'            Code und darber hinaus ist das Signal Quarzstabil und hochgenau.
'            Es gibt eine Flle von Mglichkeiten. Im Datasheet sind sie auf
'            15 Seiten beschrieben.
' Hinweis  : Im Beispiel ist der Generator als Sub ausgefhrt, damit man die
'            Software leicht im eigenen Programm verwenden kann.
' Bedienung: Taste [2/4] erhht die Ausgabefrequenz um 100 Hz
'            Taste [3/8] mindert die Ausgabefrequenz um 100 Hz
'            Taste [R] schaltet den Generator an/aus
' <db>2023-01-15
' -----------------------------------------------------------------------------
' -----------------------------------------------------------------------------
$regfile   = "m16def.dat"              ' zu jedem Modell passende Datei
'$crystal   = 7372800                  ' Quarz (Resonator) Frequenz
$crystal   = 14745600                  ' Quarzfrequenz
$baud      = 115200 '38400 '9600'      ' gewnschte Baudrate vorgeben
$hwstack   = 32
$swstack   = 32
$framesize = 64

' ========================= Dimensionierungen =================================

Dim BoGlob  as Boolean                 ' 1Bit Speicher  0-1
Dim ByGlob  as Byte                    ' 1Byte Speicher 0-256
Dim IgGlob  as Integer                 ' 2Byte -32768 bis 32767
Dim WoGlob  as Word                    ' 2Byte 0-65535 (DWord=4Byte)
Dim DwGlob  as DWord                   ' 4Byte
Dim SiGlob  as Single                  ' 4Byte >2Miliarden mit Kommastellen
Dim StGlob  as String * 30             ' 2Byte +1Byte pro weiterem Zeichen

' ========================= Konstanten ========================================
Const An   = 1
Const Aus  = 0
' ===================== Ports parametrieren =====================================

'S1 Alias Pind.5 : DDRD.5=0: PortD.5=1  ' Taste [S] (Eingang)
S2 Alias Pind.2 : DDRD.2=0: PortD.2=1  ' Taste [2] (Eingang)
S3 Alias Pind.3 : DDRD.3=0: PortD.3=1  ' Taste [3] (Eingang)
S4 Alias Pind.4 : DDRD.4=0: PortD.4=1  ' Taste [R] (Eingang)
U1 Alias PinA.6 : DDRA.6=0: PortA.6=0  ' Analogeingang (Kanal 6) Uin
U2 Alias PinA.7 : DDRA.7=0: PortA.7=0  ' Analogeingang (Kanal 7) 5Volt
B1 Alias PortB.3: DDRB.3=1: PortB.3=0  ' Beep (Piezo)
V7 Alias Portb.4: DDRB.4=1: PortB.4=1  ' rechte LED der Viererkette V4-V7
V6 Alias Portb.5: DDRB.5=1: PortB.5=1  ' 2. LED von links nach rechts//MOSI
V5 Alias Portb.6: DDRB.6=1: PortB.6=0  ' 3. LED                      //MISO
V4 Alias Portb.7: DDRB.7=1: PortB.7=1  ' linke LED der Viererkette   //SCK
Ho Alias Portb.0: DDRB.0=1: PortB.0=1  ' Hold fr Ein/Aus Mimik (Ausgang)
Of Alias Portb.1: DDRB.1=0: PortB.1=0  ' Off fr Ein/Aus Mimik  (Eingang)


Print CHR(12) : Print
Print "File: "; Version(3)
Print "Date: "; Version(1)


'----- Timer1 als Frequenzgenerator configurieren Tastung fest bei 50/50 ------
'(
TCCR1A = &B0100_0000
'          |||| ||||
'          |||| |||+- WGM10 = 0 Waveform Generation Mode; Mode 14 (fast PWM)
'          |||| ||+-- WGM11 = 1 Waveform Generation Mode; Mode 14 (fast PWM)
'          |||| |+--- FOC1B = X force output compare fr Kanal A; [1]
'          |||| +---- FOC1A = X force output compare fr Kanal B; [1]
'          |||+------ COM1B0= 0 toggle PortD.4(OC1B),lsche bei TOP
'          ||+------- COM1B1= 0 toggle PortD.4(OC1B),lsche bei TOP
'          |+-------- COM1A0= 1 toggle PortD.5(OC1A)bei compare match
'          +--------- COM1A1= 0 toggle PortD.5(OC1A) bei compare match
'             -----------Datasheet Seite 104--------------------
TCCR1B = &B0000_1010
'          |||| ||||
'          |||| |||+- CS10 = 1  Prescaler auf 1, Timer an (No prescaling)
'          |||| ||+-- CS11 = 0  Prescaler 010 = Timer an prescaling /8
'          |||| |+--- CS12 = 0  Prescaler 011 = /64 oder 100 = /256
'          |||| +---- WGM12= 1  WaveformGenerationMode:15 (FPWM, TOP = OCR1A)
'          |||+------ WGM13= 1  WaveformGenerationMode:15 (FPWM, TOP = OCR1A)
'          ||+------- n.b. = 0  nicht belegt
'          |+-------- ICES1= 0  InputCaptureEdgeSelect - hier ohne Bedeutung
'          +--------- ICNC1= 0  InputCaptureNoiseCanceller; nicht einschalten
'
' --------------nur zur INfo und experimentieren-------------------------------
'       ist nur fr Interrupt, was aber den Timer1 langsamer macht
TIMSK = &B0000_0000
'         |||| ||||
'         |||| |||+- TOIE0 = x  betrifft Timer 0
'         |||| ||+-- OCIE0 = x  betrifft Timer 0
'         |||| |+--- TOIE1 = 0  Timer 1 overflow interrupt enable - An
'         |||| +---- OCIE1B= 0  Output Compare Match B Interrupt - Aus
'         |||+------ OCIE1A= 0  Output Compare Match A Interrupt - Aus
'         ||+------- TICIE1= 0  Input capture Interrupt Enable - Aus
'         |+-------- TOIE2 = x  betrifft Timer 2
'         +--------- OCIE2 = x  betrifft Timer 2
')


Declare Sub FreqGenerator (ByVal Hz as Word, ByVal OnOff as Byte)

IgGlob = 400                    ' Eingabe in Grundeinheit (Hz)
Call FreqGenerator (IgGlob,An)

'----------------------------Hauptroutine------------------------------------
Do

  'Debounce S1 , 0 , Tast1 , Sub        ' S1 ["S"]
  Debounce S2 , 0 , Tast2 , Sub        ' S2 ["2"]
  Debounce S3 , 0 , Tast3 , Sub        ' S3 ["3"]
  Debounce S4 , 0 , Tast4 , Sub        ' S4 ["R"]

Loop



'--------------------Tastenroutine--------------------------------------------
Tast1: ' [S]
  'V6 = Not V6                         ' Der Zustndige Pin ist belegt durch
  'Sound B1,150,500                    ' PortD.5(OC1A)
Return

'-----------------------------------------------------------------------------
Tast2: ' [2/4]
  V4 = Not V4                          ' LED V7 ndert seinen Zustand
  IgGlob = IgGlob + 100
  V5 = 0
  Call FreqGenerator (IgGlob,an)

Return

'-----------------------------------------------------------------------------
Tast3: ' [3/8]
  V7 = Not V7                          ' LED V4 ndert seinen Inhalt
  IgGlob = IgGlob - 100
  V5 = 0
  Call FreqGenerator (IgGlob,1)

Return
'-----------------------------------------------------------------------------
Tast4: ' [R]
  V5= Not V5
  'TCCR1A.6 = not TCCR1A.6               ' Timer1 An=1 Aus=0
  Call FreqGenerator (IgGlob,Not V5)
Return





'Aufgabe   : Frequenzausgabe mit Hilfe von Timer1 ber PortD.5 (OC1A)
'Parameter : Hz eine Zahl von 1Hz bis 60KHz
'            OnOff schaltet Generator an=1 aus=0
'Ausgabe   : PortD.5 (OC1A) am AVRKit D-Port X20, Pin2
'Hinweis   : An/Aus geht auch direkt ber
'Lend     : 2023-01-15
Sub FreqGenerator (ByVal Hz as Word, ByVal OnOff as Byte)

  Local FGCompare as DWord
                                         ' der Compare-Wert berechnet sich aus:
  FGCompare = _XTAL / 16                 ' Quarzfrequenz/8/2/Hz. Die zustzliche
  FGCompare = FGCompare / Hz             ' Teilung durch 2 ist erfordelich, weil
                                         ' erst zwei Compare Ereignisse eine
                                         ' Vollwelle ergeben.
  DDRD.5    = 1                          ' Pin PD5/OC1A ist Ausgang (Atmega16)
  TCCR1A    = &B0100_0000                ' Pin OC1A togglen, kein PWM
  TCCR1B    = &B0000_1010                ' Timer-Reset bei Compare, Clock/8
  COMPARE1A = FGCompare
  TCCR1A.6 = OnOff.0

  Print "Herz     : "; Hz; " (Hz)"       ' Die Printausgabe dient lediglich
  Print "FGCompare: "; FGCompare         ' zum Testen und sollte auskommentiert
  Print                                  ' werden.

End Sub


